@@ -140,6 +140,29 @@ class Module:
             cb(data)
         self.subscribe_mqtt("$iothub/twin/PATCH/properties/desired/+", handler)
 
+    async def update_telemetry(self, data):
+        id = str(self.rid)
+        self.rid += 1
+        filter = f"$iothub/twin/res/+/?$rid={id}"
+        future = asyncio.get_running_loop().create_future()
+
+        def handler(c, u, message: mqtt.MQTTMessage):
+            topic: str = message.topic
+
+            code = topic.split("/")[3]
+            future.set_result(code)
+
+        self.client.message_callback_add(filter, handler)
+
+        self.publish_mqtt(
+            f"$iothub/twin/PATCH/properties/reported/?$rid={id}", json.dumps(data))
+
+        while not future.done():
+            await asyncio.sleep(.1)
+
+        self.client.message_callback_remove(filter)
+        return future.result()
+
     async def get_twin(self):
         if not self.client:
             raise error("Client not initialized")
@@ -174,19 +197,20 @@ class Module:
         return future.result()
 
     def on_method(self, name: str, cb):
-        def handler(topic: str, payload):
-            rid = topic.split("=")[1]
-
+        def handler(topic: str, payload: bytes):
+            [notused, rid] = topic.split("=")
             try:
-                args = json.loads(payload)
-                result = cb(args)  # TODO: make it async
+                data = json.loads(payload.decode())
+                result = cb(data)
                 self.publish_mqtt(
-                    f"$iothub/methods/res/200/?$rid={rid}", json.dumps(result))
-            except:
+                    f"$iothub/methods/res/{200}/?$rid={rid}", json.dumps(result))
+            except Exception as e:
+                print(f"Error when handling method {name}: {e}", flush=True)
                 self.publish_mqtt(
-                    f"$iothub/methods/res/500/?$rid={rid}", 'error')
+                    f"$iothub/methods/res/{500}/?$rid={rid}", json.dumps("error"))
+                pass
 
-        self.subscribe_mqtt(f"$iothub/methods/POST/{name}/#", handler)
+        self.subscribe_mqtt(f"$iothub/methods/POST/{name}/+", handler)
 
     def subscribe(self, type: str, cb):
         if not '/' in type:
